#define PCF8574_ADDR 0x40

byte PCF8574DataIn   = 0xFF;
byte PCF8574DataOut  = 0xFF;


//----------------------------------------------------------
// KnopUp - Return true als knop 'omhoog' ingedrukt
//----------------------------------------------------------
//----------------------------------------------------------
bool KnopUp()
{
  return ! (PCF8574DataIn & 0x20);
}

//----------------------------------------------------------
// KnopDown - Return true als knop 'omlaag' ingedrukt
//----------------------------------------------------------
//----------------------------------------------------------
bool KnopDown()
{
  return ! (PCF8574DataIn & 0x10);
}

//----------------------------------------------------------
// PCF8574Read - lees ingangen van i2c io chip
//----------------------------------------------------------
// Resultaat in globale variabele PCF8574DataIn
//----------------------------------------------------------
void PCF8574Read()
{  
  if (i2c_start(PCF8574_ADDR | I2C_READ)) {
    PCF8574DataIn = i2c_read(true);  // only 1 = last byte
    i2c_stop();
//       printf("PCF 0x%02x\n", PCF8574DataIn);
  } 
  else {
    i2c_stop();
    printf("PCF8574 i2c read fail\n");
  }
}

//----------------------------------------------------------
// PCF8574Write - schrijf uitgangen van i2c io chip
//----------------------------------------------------------
// Data komt uit globale variabele PCF8574DataIOut
//----------------------------------------------------------
void PCF8574Write()
{  
  if (i2c_start(PCF8574_ADDR | I2C_WRITE)) {
    i2c_write(PCF8574DataOut);
    i2c_stop();
  } 
  else {
    i2c_stop();
    printf("PCF8574 i2c write fail\n");
  }
}


//----------------------------------------------------------
// FreeRam - actueel vrij geheugen in bytes.
//----------------------------------------------------------
//----------------------------------------------------------
int freeRam () {
  extern int __heap_start, *__brkval; 
  int v; 
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
}


bool LedFlashMode;
long int LedColor1, LedColor2;

//----------------------------------------------------------
// LedTakt - uitvoering van LedColor / LedBlink / LedFlash
//----------------------------------------------------------
//----------------------------------------------------------
void LedTakt()
{  
  static byte Counter;

  Counter ++;
  if (Counter > 50) {
    // Halve seconde
    Counter = 0;  
    leds.setPixelColor(0, LedColor1);
    leds.show();     
  }
  if (
  ((LedFlashMode == true)  && (Counter == 1)) ||
    ((LedFlashMode == false) && (Counter == 25))  ) {
    leds.setPixelColor(0, LedColor2);
    leds.show();          
  }
}

//----------------------------------------------------------
// LedColor - Led aan in 1 vaste kleur
//----------------------------------------------------------
//----------------------------------------------------------
void LedColor(long Color) 
{
  //  printf("LedColor %ld\n", Color);
  if (!BatterijLowFlag) {
    LedColor1 = Color;
    LedColor2 = Color;
  }
}

//----------------------------------------------------------
// LedFlash - Flash led in kleur + zwart (10%? duty cycle)
//----------------------------------------------------------
//----------------------------------------------------------
void LedFlash(long Color1) 
{
  LedFlash(Color1, BLACK);
}

//----------------------------------------------------------
// LedFlash - Flash led in twee kleuren (10%? duty cycle)
//----------------------------------------------------------
//----------------------------------------------------------
void LedFlash(long Color1, long Color2) {
//  printf("LedFlash %ld %ld\n", Color1, Color2);

  if (!BatterijLowFlag) {
    LedColor1 = Color1;
    LedColor2 = Color2;
    LedFlashMode = true;
  }
}

//----------------------------------------------------------
// LedBlink - Knipper led in kleur + zwart (50% duty cycle)
//----------------------------------------------------------
//----------------------------------------------------------
void LedBlink(long Color1) 
{
  LedBlink(Color1, BLACK);
}

//----------------------------------------------------------
// LedBlink - Knipper led in twee kleuren (50% duty cycle)
//----------------------------------------------------------
//----------------------------------------------------------
void LedBlink(long Color1, long Color2) 
{
  //   printf("LedBlink %ld %ld\n", Color1, Color2);

  if (!BatterijLowFlag) {
    LedColor1 = Color1;
    LedColor2 = Color2;
    LedFlashMode = false;
  }
}


//----------------------------------------------------------
// ProgrammaKleur - return color code of selected program
//----------------------------------------------------------
//----------------------------------------------------------
long ProgrammaKleur()
{
  switch (ProgrammaNummer) {

  case 0 : 
    return YELLOW; 
  case 1 : 
    return DARKGREEN; 
  case 2 : 
    return DARKBLUE; 
  case 3 : 
    return PURPLE; 
  default : 
    return DARKRED;
  }
}


