
#define LIJNSENSOR_LED 8

// i2c pins
// SDA 11
// SCL 12

// NeoPixel setup
#define LED_PIN 13
#define LED_COUNT 1

#include <Adafruit_NeoPixel.h> 
#include "WS2812_Definitions.h" 
// Create an instance of the Adafruit_NeoPixel class called "leds".
// That'll be what we refer to from here on...
Adafruit_NeoPixel leds = Adafruit_NeoPixel(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);

//---------------------------------------------------------
// Gebruik van software i2c, zodat de 'hardware i2c' pinnen
// beschikbaar blijven als analoge input.
// http://playground.arduino.cc/Main/SoftwareI2CLibrary
#define SDA_PORT PORTB  // Digital 11 = PB3
#define SDA_PIN 3
#define SCL_PORT PORTB  // Digital 12 = PB4
#define SCL_PIN 4
#define I2C_TIMEOUT 1000  // ms
#include <SoftI2CMaster.h>
#include <avr/io.h>

int BatterijSpanning;
bool BatterijLowFlag = false;

int ProgrammaNummer = 0;

// to support printf 
int my_putc(char c, FILE *t) {  
  if (c == '\n') Serial.write('\r');
  return Serial.write(c);
}

//----------------------------------------------------------
// setup -
//----------------------------------------------------------
//----------------------------------------------------------
void setup() {
  Serial.begin(115200);          // start serial
  fdevopen( &my_putc, 0);        // device 0 (stdout) output naar my_putc()

  LedFlash(ProgrammaKleur(), BLACK);

  pinMode(LIJNSENSOR_LED, OUTPUT);

  i2c_init();

  printf("\n");
  BatterijTakt();  
  PrintStatus();
  printf("Lijnrobot ready.\n");
}


//----------------------------------------------------------
// loop -
//----------------------------------------------------------
//----------------------------------------------------------
void loop() {
  static int MainTakt, SecTakt;
  
  // 10 ms lus
  int us = micros();
  if ((us - MainTakt) > 0) {
    MainTakt = us + 10000;  // zet tijd voor volgende interval 

    PCF8574Read();  // lees status knopjes
    LijnSensorTakt();  
    RijdenTakt();  
    PCF8574Write();  // schrijf IO (motoren) (niet urgent; dit betreft alleen rijrichting; pwm gaat direct.

    // lees onderste knop    
    static bool VorigeKnop;
    bool ActKnop = KnopDown();
    if (ActKnop != VorigeKnop) {
      // flank 
      VorigeKnop = ActKnop;  

      if (ActKnop) {
        // knop ingedrukt
        ProgrammaNummer ++;
        if (ProgrammaNummer > 3) ProgrammaNummer = 0;
        LedFlash(ProgrammaKleur(), BLACK);

        printf("Knop %d %d %ld\n", ActKnop, ProgrammaNummer, ProgrammaKleur());
      }
    }
    LedTakt();
    BatterijTakt();
    
//    // controle takt-tijd
//    int Delta = (int)micros() - us;
//    printf("Takt tijd: %d us\n", Delta);
  }

  // seconde lus
  int ms = millis();
  if ((ms - SecTakt) > 0) {
    SecTakt = ms + 1000;  // zet tijd voor volgende interval 

    // controleer de batterijspanning (in mV)
    if ((BatterijSpanning != 0) && (BatterijSpanning < 6500)) {
      // battery low
      LedFlash(DARKRED, BLACK);
      BatterijLowFlag = true;
    }
  }      
}

//----------------------------------------------------------
// BatterijTakt - Bepaal gemiddelde Batterijspanning.
//----------------------------------------------------------
//----------------------------------------------------------
void BatterijTakt()
{  static long BatterijGemiddelde;
   long B;

  // bepaal batterijspanning (in mV)
  B = analogRead(A5) * 975L / 100;

  if (BatterijGemiddelde == 0) {
    // eerste keer
    BatterijGemiddelde = B * 128L;
  }
  
  // Gemiddelde batterijs
  BatterijGemiddelde  += (B - BatterijGemiddelde / 128);
  BatterijSpanning = BatterijGemiddelde / 128;
}

//----------------------------------------------------------
// PrintStatus
//----------------------------------------------------------
//----------------------------------------------------------
void PrintStatus()
{
   printf("Batterij: %d mV\n", BatterijSpanning);
   printf("Ram: %d bytes free\n", freeRam());  
}


