// StateMachine.ino - 

static int StartTijd;

//-----------------------------------------------------------------------------
// RijdenTakt - State machine voor aansturing motor.
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void RijdenTakt()
{ 
  static int State, PrevState = -1;
  static int Delay, DraaiTijd;
  static int SpeedSP, SpeedSP2; // SP = actuele snelheid die opbouwt naar SP2 = gewenste snelheid
  static int LijnGeldig;

  bool NewState = false;

  // rapporteer status bij state overgang
  if (PrevState != State) {  
    PrevState = State;  
    printf("RijdenTakt State: %d\n", State);
    NewState = true;
  }

  // zorg dat SpeedSP (langzaam) SpeedSP2 volgt
  if (SpeedSP < SpeedSP2) {
    SpeedSP += 6;
    if (SpeedSP > SpeedSP2) SpeedSP = SpeedSP2;
  } 
  else {
    SpeedSP -= 10;
    if (SpeedSP < SpeedSP2) SpeedSP = SpeedSP2;
  }


  // state machine
  switch(State) {
  case 0 : // State: rust
    {  
      if (NewState) {
        LedFlash(ProgrammaKleur());
      }

      // State actie
      Motors(0,0);

      // Condities
      if (KnopUp()) { // Knop ingedrukt
        State = 10;  // rijd tot object
      }
      break;
    }  

  case 10 :       // State actie : start-vertraging
    {
      if (NewState) {
        LedBlink(ProgrammaKleur());
        Delay = 100;
        PrintStatus();
      }
      Delay --;

      if (!Delay) {
        LedColor(ProgrammaKleur());
        State = 1000 * (ProgrammaNummer + 1);    
        StartTijd = millis();
      }
      break;
    }

    // Programma 0        
  case 1000 :       // State actie : draai naar links (calibratie)
    {
      if (NewState) {
        // bepaal tijdsduur draai
        DraaiTijd = 50;
        if (ProgrammaNummer == 0) LsCalibrationStart();
      }

      Motors(-25, 30);
      LijnSensorPrint();
      if (DraaiTijd) DraaiTijd --;
      if (DraaiTijd == 0) {
        if (ProgrammaNummer == 0) LsCalibrationEnd();
        State = 111;  // klaar
      }
      break;
    }


    // Programma 1 - controle calibratie (zit verweven in state 1000 en verder)
  case 2000 : 
    {
      State = 1000;
      break;
    }


    // programma 2  - volg lijn 
  case 3000 :
      {
         if (NewState) {
            SpeedSP2 = 210;  // gewenste snelheid; SpeedSP volgt
            SpeedSP = 0;
            LijnGeldig = 5;  // initieel tegoed, waarde niet zo belangrijk   
         }
      
         int Correctie = 17 * (LijnSensorOutput - 6);  
         int SpeedL, SpeedR;
         SpeedL = SpeedSP + Correctie;
         SpeedR = SpeedSP - Correctie;    
         
         // Als 1 moter buiten regelbereik komt, correctie op de andere overnemen.
         if (SpeedL > 255) {
            SpeedR = SpeedR + 255 - SpeedL;
            SpeedL = 255;
         }
         if (SpeedR > 255) {
            SpeedL = SpeedL + 255 - SpeedR;
            SpeedR = 255;
         }
         Motors(SpeedL, SpeedR);                
         printf("Q %d %d\n", LijnSensorOutput, LijnSensorValid);

         // einde bereikt?      
         if (LijnSensorValid) {
            LijnGeldig = 40;  // dit bepaalt hoe lang we zonder lijn rijden voordat we stoppen.
         } else {
            // aftellen.
            if (LijnGeldig) LijnGeldig --;
            
            // Als we de lijn in het midden kwijt zijn, snel stoppen
            if ((LijnSensorOutput > 1) && (LijnSensorOutput < 10)) {
              LijnGeldig = 0;
            }
            
         }
         if (LijnGeldig == 0) {
            State = 119;  
         }
         break;
      }

  default : 
    {
      printf("RijdenTakt: ongeldige state %d (tijd %d ms)\n", State, (int)(millis() - StartTijd));
      Motors(0, 0);
      State = 0;
      break;
    }
  } // einde van switch 
}










